#!/usr/bin/python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/

"""
The MIT License (MIT)

Copyright (c) 2014 Al Audet

Permission is hereby granted, free of charge, to any person obtaining a copy
of Raspi-Sump and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
"""

import time
import decimal
import smtplib
import string
filename = "/home/al/pump_failure-{}.csv".format(time.strftime("%Y%m%d"))


def water_distance():
    """Measure the distance of water using the HC-SR04 Ultrasonic Sensor."""
    critical_distance = 35
    start_level = 30.0
    water_rising = 0.5
    print "When water level reaches {} cm's generate an email alert".format(
        critical_distance
        )
    time.sleep(3)

    try:
        while True:
            start_level += water_rising
            sample = [start_level for i in range(11)]
            handle_error(sample, critical_distance, filename)
    except KeyboardInterrupt:
        print "Script killed by user"


def handle_error(sample, critical_distance, filename):
    """Eliminate fringe error readings by using the median reading of a
    sorted sample."""
    sorted_sample = sorted(sample)
    true_distance = sorted_sample[5]
    capture = open(filename, 'a')

    if true_distance > critical_distance:
        smtp_alerts(true_distance, capture)
    else:
        level_good(true_distance, capture)


def level_good(how_far, target):
    """Process reading if level is greater than critical distance."""
    decimal.getcontext().prec = 3
    how_far_clean = decimal.Decimal(how_far) * 1
    print how_far_clean,
    print "Water Level Rising"
    target.write(time.strftime("%H:%M:%S,")),
    target.write(str(how_far_clean)),
    target.write("\n")
    target.close()
    time.sleep(1)


def smtp_alerts(how_far, target):
    """Process reading and generate alert if less than critical distance."""
    # username = "your smtp username here "
    # password = "your smtp password here"
    smtp_server = "localhost:25"

    decimal.getcontext().prec = 3
    how_far_clean = decimal.Decimal(how_far) * 1

    target.write(time.strftime("%H:%M:%S,")),
    target.write(str(how_far_clean)),
    target.write("\n")
    target.close()

    email_from = "Raspi-Sump <email@yourdomain.com>"
    email_to = "email@yourdomain.com or wireless sms email for sms alerts"
    email_body = string.join((
        "From: {}".format(email_from),
        "To: {}".format(email_to),
        "Subject: Sump Pump Alert!",
        "",
        "Critical! The sump pit water level is {} cm.".format(str(
            how_far_clean)),), "\r\n"
        )
    print ""
    print "Pump Failure!!! Critical Level Breached. Email Sent"
    time.sleep(1)
    print ""
    print email_from
    print email_to
    print email_body
    server = smtplib.SMTP(smtp_server)
    # server.starttls()
    # server.login(username, password)
    server.sendmail(email_from, email_to, email_body)
    server.quit()
    exit(0)

if __name__ == "__main__":
    water_distance()



'''Class to create a measurement using HC-SR04 sensor connected to the
Raspberry Pi GPIO pins.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time
import math
import RPi.GPIO as GPIO


class Measurement(object):
    '''Create a measurement using an HC-SR04 Ultrasonic Sensor'''
    def __init__(self, trig_pin, echo_pin, rounded_to, temperature):
        self.trig_pin = trig_pin
        self.echo_pin = echo_pin
        self.rounded_to = rounded_to
        self.temperature = temperature

    def distance(self):
        """Return the distance, in cm, of an object adjusted for
        temperature in Celcius."""
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)
        speed_of_sound = 331.3 * math.sqrt(1+(self.temperature / 273.15))
        sample = []
        for distance_reading in range(11):
            GPIO.setup(self.trig_pin, GPIO.OUT)
            GPIO.setup(self.echo_pin, GPIO.IN)
            GPIO.output(self.trig_pin, GPIO.LOW)
            time.sleep(0.3)
            GPIO.output(self.trig_pin, True)
            time.sleep(0.00001)
            GPIO.output(self.trig_pin, False)
            while GPIO.input(self.echo_pin) == 0:
                sonar_signal_off = time.time()
            while GPIO.input(self.echo_pin) == 1:
                sonar_signal_on = time.time()
            time_passed = sonar_signal_on - sonar_signal_off
            distance_cm = time_passed * ((speed_of_sound * 100) / 2)
            sample.append(distance_cm)
            GPIO.cleanup()
        sorted_sample = sorted(sample)
        sensor_distance = sorted_sample[5]
        return round(sensor_distance, self.rounded_to)



'''Module to send SMTP alerts in case of sump pump failure'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import smtplib
import string
import ConfigParser

config = ConfigParser.RawConfigParser()
config.read('/home/pi/raspi-sump/raspisump.conf')

configs = {'email_to': config.get('email', 'email_to'),
           'email_from': config.get('email', 'email_from'),
           'smtp_authentication': config.getint(
               'email', 'smtp_authentication'),
           'smtp_tls': config.getint('email', 'smtp_tls'),
           'smtp_server': config.get('email', 'smtp_server'),
           'username': config.get('email', 'username'),
           'password': config.get('email', 'password')
           }


def smtp_alerts(water_depth):
    """Generate email alert if water level greater than critical distance."""
    recipients = configs['email_to'].split(', ')
    email_body = string.join((
        "From: {}".format(configs['email_from']),
        "To: {}".format(configs['email_to']),
        "Subject: Sump Pump Alert!",
        "",
        "Critical! The sump pit water level is {} cm.".format(
            str(water_depth)
        ),), "\r\n"
        )

    server = smtplib.SMTP(configs['smtp_server'])
    # Check if smtp server uses TLS
    if configs['smtp_tls'] == 1:
        server.starttls()
    else:
        pass
    # Check if smtp server uses authentication
    if configs['smtp_authentication'] == 1:
        username = configs['username']
        password = configs['password']
        server.login(username, password)
    else:
        pass

    server.sendmail(configs['email_from'], recipients, email_body)
    server.quit()



'''Module to graph sump pit activity.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import rcParams
rcParams.update({'figure.autolayout': True})


def graph(csv_file, filename):
    """Create a line graph from a two column csv file."""
    date, value = np.loadtxt(csv_file, delimiter=',', unpack=True,
                             converters={0: mdates.strpdate2num('%H:%M:%S')}
                             )
    fig = plt.figure(figsize=(10, 3.5))
    fig.add_subplot(111, axisbg='white', frameon=False)
    rcParams.update({'font.size': 9})
    plt.plot_date(x=date, y=value, ls='solid', linewidth=2, color='#FB921D',
                  fmt=':'
                  )
    title = "Sump Pit Water Level {}".format(time.strftime('%Y-%m-%d %H:%M'))
    title_set = plt.title(title)
    title_set.set_y(1.09)
    plt.subplots_adjust(top=0.86)
    plt.ylabel('Centimeters')
    plt.xlabel('Time of Day')
    plt.xticks(rotation=30)
    plt.grid(True, color='#ECE5DE', linestyle='solid')
    plt.tick_params(axis='x', bottom='off', top='off')
    plt.tick_params(axis='y', left='off', right='off')
    plt.savefig(filename, dpi=72)
